import sys
import asyncio
import logging

try:
    from usody_sanitize import erasure, __version__
except ModuleNotFoundError:
    import pathlib
    sys.path.append(
        pathlib.Path(__file__).parent.parent.absolute().as_posix()
    )
    from usody_sanitize import erasure, __version__


logger = logging.getLogger(__name__)


def run():
    """Forces to run the function in a new async loop."""
    loop = asyncio.new_event_loop()

    try:
        logger.info("Starting sanitize main process.")
        erasures = loop.run_until_complete(erasure.auto_erase_disks())

    finally:
        loop.close()

    return erasures


if __name__ == '__main__':
    # Run this one for development.
    # Run it.
    logging.basicConfig(
        force=True,
        stream=sys.stdout,
        level=logging.DEBUG,
        format="%(asctime)s - %(levelname)s %(name)s -"
               " [%(funcName)s at line %(lineno)d]: %(message)s"
    )
    data = run()

    import json
    with open('/tmp/erasure_output.json', 'w') as _fh:
        json.dump(data, _fh, indent=4)

    # print(erasure.ErasureMethods.baseline_erasure)




from typing import Optional
from pydantic import BaseModel, Field

from .export_data import ExportData


class Device(BaseModel):
    """Information of the device."""
    manufacturer: Optional[str] = Field(default=None)
    model: Optional[str] = Field(default=None)
    serial_number: Optional[str] = Field(default=None)
    storage_medium: Optional[str] = Field(default=None,
                                          description="HDD/SSD/SSDHD")
    connector: Optional[str] = Field(default=None,
                                     description="IDE/SATA/SCSI/SAS/M.2/U.2")
    size: Optional[int] = Field(default=None, description="Disk size in bites")

    export_data: ExportData = Field(default=ExportData)



import time
import logging

from usody_sanitize import schemas, commands, utils

logger = logging.getLogger(__name__)


async def erase_ssd_hdparm(dev_path: str) -> schemas.ErasureStep:
    """
    Generates a erasure step for deleting SSD using hdparm.

    :param str dev_path: Path to the device.
    :return: schemas.ErasureStep

    Example:
    >>> erase_ssd_hdparm("/dev/sda")
    """
    step = schemas.ErasureStep(device=dev_path, step=1)
    step.date_init = time.time()

    # Start first command for this step.
    logger.debug(f"{dev_path}: Start command 1.")
    command1 = f"hdparm -I {dev_path}"
    cmd1: schemas.ErasureCommand = await commands \
        .erasure_command(command1)
    cmd1.description = "Verify that the SSD disc is not frozen."

    if utils.find_text("(not[\t ]*frozen)", cmd1.stdout):
        cmd1.success = True
    else:
        cmd1.success = False

    step.commands.append(cmd1)
    logger.debug(f"{dev_path}: Command 1 finished. \n{cmd1}")
    if cmd1.success is False:
        return step

    # Second command for this step.
    logger.debug(f"{dev_path}: Start command 2.")
    command2 = f"hdparm --user-master u --security-set-pass Usody {dev_path}"
    cmd2 = await commands.erasure_command(command2)
    cmd2.description = "Set a temporal password to lock the device."

    cmd2.success = cmd2.return_code == 0

    step.commands.append(cmd2)
    logger.debug(f"{dev_path}: Command 2 finished. \n{cmd2}")
    if cmd2.success is False:
        logger.warning("")
        return step

    # Third command.
    logger.debug(f"{dev_path}: Start command 3.")
    command3 = f"hdparm --user-master --security-erase Usody {dev_path}"
    cmd3 = await commands.erasure_command(command3)
    cmd3.description = "Erase the SSD changing the encryption key."
    cmd3.success = cmd3.return_code == 0 or cmd3.return_code == 22

    step.commands.append(cmd3)
    logger.debug(f"{dev_path}: command 3 finished. \n{cmd3}")
    if cmd3.success is False:
        return step

    # Fourth command.
    logger.debug(f"{dev_path}: Start command 4.")
    command4 = f"hdparm -I {dev_path}"
    cmd4 = await commands.erasure_command(command4)
    cmd4.description = "Check the drive security is set to disabled"
    # There is no problem if the drive is locked. A validation
    # process of the data been erased is more important.
    # if utils.find_text(" *(not *enabled) *", cmd4.output):
    #     cmd4.success = True
    # else:
    #     cmd4.success = False

    # @TODO: Validation step at the end.

    step.commands.append(cmd4)
    logger.debug(f"{dev_path}: Command 4 finished. {cmd4}")

    # Write final values on the step schema.
    step.date_end = time.time()
    step.duration = step.date_end - step.date_init
    step.success = all(cmd.success for cmd in step.commands)

    logger.debug(f"{dev_path}: hdparm erasure step finished.")
    return step


async def erase_hdd_shred(dev_path: str) -> schemas.ErasureStep:
    """Runs erasure step for deleting HDD using shred.

    Shred is a command line utility for securely deleting files, it
    is the program used for the "Basic" erasure method.

    :param str dev_path: Path to the device.
    :return: schemas.ErasureStep

    Example:
    >>> erase_hdd_shred("/dev/sda")
    """
    step = schemas.ErasureStep(device=dev_path, step=1)
    step.date_init = time.time()

    # Define the command to run, with zeros or random.
    command = f"shred --force --verbose --zero --iterations=0 {dev_path}"
    logger.debug(f"{dev_path} command: {command}")

    # Run the command.
    cmd: schemas.ErasureCommand = await commands.erasure_command(
        command=command, process_manager=utils.print_shred_progress)
    cmd.description = "Write zeros to the disk with `shred`."

    # Write final values on the step schema.
    step.commands.append(cmd)
    step.date_end = time.time()
    step.duration = step.date_end - step.date_init
    step.success = all(cmd.success for cmd in step.commands)

    logger.debug(f"{dev_path}: Badblocks erasure step finished.")
    return step


async def erase_hdd_badblocks(dev_path: str) -> schemas.ErasureStep:
    """Runs erasure step for deleting HDD using `badblocks`.

    badblocks will delete the disk writting data into each sector.

    :param str dev_path: Path to the device.
    :return: schemas.ErasureStep

    Example:
    >>> erase_hdd_badblocks("/dev/sda")
    """
    step = schemas.ErasureStep(device=dev_path, step=1)
    step.date_init = time.time()

    # Todo: Add -e argument to add a maximum of badblocks found.
    # Define the command to run, with zeros or random.
    # Argument `-s` provides a progress info that cannot be received,
    #  causing a exception as no new lines are emitted.
    command = f"badblocks -wv -p 1 -t random {dev_path}"
    logger.debug(f"{dev_path} command: {command}")

    # Run the command.
    cmd: schemas.ErasureCommand = await commands.erasure_command(
        command=command, process_manager=utils.print_badblocks_progress)
    cmd.description = "Write random data into the disk with `badblocks`."

    # Write final values on the step schema.
    step.commands.append(cmd)
    step.date_end = time.time()
    step.duration = step.date_end - step.date_init
    step.success = all(cmd.success for cmd in step.commands)

    logger.debug(f"{dev_path}: Command badblocks erasure step finished.")
    return step




import sys
import json
import logging
import asyncio
import argparse

try:
    from usody_sanitize.erasure import DefaultMethods, auto_erase_disks
except ModuleNotFoundError:
    import pathlib
    sys.path.append(
        pathlib.Path(__file__).parent.parent.absolute().as_posix()
    )
    from usody_sanitize.erasure import DefaultMethods, auto_erase_disks

from usody_sanitize import __version__ as app_version


logging.getLogger("CMD")


def run_cmd():
    args = parse_args()
    configure_loggers(args.log_level)

    # Run erasures.
    result = run_coroutine(auto_erase_disks(args.method, args.device))

    # Todo: Add function to handle the result exports.
    logging.debug(json.dumps(result, indent=4))
    with open('/tmp/erasure_output.json', 'w') as _fh:
        json.dump(result, _fh, indent=4)


def configure_loggers(level="INFO"):
    logging.basicConfig(
        force=True,
        stream=sys.stdout,
        level=getattr(logging, level.upper()),
        format="%(asctime)s - %(levelname)s: %(message)s"
    )


def run_coroutine(coro):
    """Forces to run the function in a new async loop."""
    loop = asyncio.new_event_loop()

    try:
        return loop.run_until_complete(coro)
    finally:
        loop.close()


def parse_args():
    parser = argparse.ArgumentParser(description='sanitize a disk')
    parser.add_argument('-m', '--method', type=str, help='sanitize method',
                        choices=['BASIC', 'BASELINE', 'ENHANCED'])

    # Select the
    disk = parser.add_mutually_exclusive_group(required=True)
    disk.add_argument('-d', '--device', type=str, action='append',
                      help='path to the /dev/{disk} E.G.: /dev/sda')
    disk.add_argument('-a', '--all', action='store_true',
                      help='all disks unless the disk mounted as root')

    parser.add_argument('--confirm', action='store_const', const=True,
                        help='confirm to sanitize disks before proceed')

    parser.add_argument('--version', action='version', version=app_version,
                        help='show the version of usody_sanitize')

    parser.add_argument('-l', '--log-level', dest='log_level', default='INFO',
                        choices=['DEBUG', 'INFO', 'WARNING', 'ERROR',
                                 'CRITICAL'],
                        help='set the logging level (default: %(default)s)')

    # Todo: Add output/export option.

    return parser.parse_args()


if __name__ == '__main__':
    run_cmd()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['usody_sanitize', 'usody_sanitize.schemas']

package_data = \
{'': ['*']}

install_requires = \
['pydantic>=1.10.2,<2.0.0']

entry_points = \
{'console_scripts': ['sanitize = usody_sanitize.cmd_client:run_cmd']}

setup_kwargs = {
    'name': 'usody-sanitize',
    'version': '0.1.0b7',
    'description': 'A tool to securely erase/wipe data on disks HDD and SSD with a proper sanitization process.',
    'long_description': '# Usody Sanitize\n\n> Under development.\n\nThis tool securely erases disks by performing a certificate-based validation of\nthe wipe process. It ensures that the data on the disk is completely and \nirrecoverably erased, protecting sensitive information from being recovered. \nThe tool uses industry-standard wiping methods to securely erase the data on \nthe disk, making it impossible to recover. The tool also generates a \ncertificate of sanitize process that can be used to verify the authenticity of the wipe\nprocess. This tool is perfect for businesses and individuals who need to\nsecurely and permanently remove sensitive data from their disks.\n\n## Todo\n\n- Test the command with more errors on erasures.\n- Better command output handler.\n- Improve the export of the erasures into json files.\n\n## Installation\n\nInstall the package from the official PyPi repository:\n\n<div class="termy">\n\n```console\n$ pip install usody_sanitize\n\n---> 100%\n```\n\n</div>\n\n## Usage\n\nYou can use this module via terminal or calling it from an external code.\n\n### Terminal client\n\nErase a single disk using the default method: \n\n```bash\nusody_sanitize -v /dev/sda  \n```\n\n\n## Issues\n\n### hdparm Error: The running kernel lacks CONFIG_IDE_TASK_IOCTL support for this device\n\nWhen trying to run the command \n`hdparm --user-master u --security-erase UsodyPassword /dev/sdX`, an error\nmessage was encountered:\n\n    The running kernel lacks CONFIG_IDE_TASK_IOCTL support for this device.\n\nA return code of 22 from the `hdparm` command generally indicates \nthat the command completed successfully, but some features may not have been \nfully supported on the device being used. This can happen if the device does \nnot fully comply with the ATA standard, or if the device is a SCSI drive rather\nthan an ATA drive. It may also indicate that the specific options used in the\ncommand are not supported on the device.\n\nThis error message is indicating that the kernel (the core part of the \noperating system) does not have support for the IDE task IOCTL feature, which\nis required to run the `hdparm` command.\n\nThe `hdparm` command is used to configure and retrieve information about ATA \nhard drives, and the specific option used in the command (`--user-master u \n--security-erase UsodyPassword`) is used to erase the security settings on the\ndrive. Because the kernel does not have the necessary support, the command\ncannot be executed.\n\nTo enable the CONFIG_IDE_TASK_IOCTL support in the kernel, it must be \nrecompiled with this option enabled. The process for doing this will depend on\nthe specific distribution of Linux being used, but generally the steps are:\n\n1. Download the source code for the current kernel version.\n2. Extract the source code and navigate to the root directory.\n3. Run `make menuconfig` or `make xconfig` to open the kernel configuration\nmenu.\n4. Search for the option `CONFIG_IDE_TASK_IOCTL` and enable it.\n5. Save the configuration and exit the menu.\n6. Run make to compile the kernel with the new configuration.\n7. Install the new kernel, and reboot the system to use it.\n\nIt\'s important to note that recompiling the kernel is a complex and delicate\nprocess and it\'s recommended to have experience with Linux kernel compilation.\n\n#### What is `CONFIG_IDE_TASK_IOCTL`\n\nIs a kernel configuration option that enables support for the IDE task file\nregister IOCTLs in the Linux kernel. The IDE task file register is a set of\nregisters on an IDE (Integrated Drive Electronics) hard drive that are used to\ncontrol the drive\'s operations.\n\nWhen this feature is enabled, the kernel provides an interface for user-space \nprograms, such as `hdparm`, to access these registers and perform various \noperations on the hard drive, such as reading and setting parameters, \nperforming security commands, and reading SMART data.\n\nIf this feature is not enabled, the `hdparm` command will not be able to access \nthe task file register and will not be able to perform certain operations on \nthe drive, such as security commands or SMART data.\n\nIt\'s worth noting that this feature is not only specific to `hdparm`, but also \nother utilities that can access the task file register, such as smartctl and \nhdparam will also be affected by the state of this feature.\n\nIt\'s also worth noting that this feature is specific to ATA drives and will not\nhave any effect on SCSI drives.\n',
    'author': 'blkpws',
    'author_email': 'me@blkpws.xyz',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/usody/sanitize',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import re
import asyncio
import logging

from typing import Optional
from usody_sanitize import schemas

logger = logging.getLogger(__name__)


def get_spaced_numbers(max_num: int, items: int):
    """
    ```python
    step = (100000 - 0) / (6 - 1)
    numbers = [i * step for i in range(6)]
    ```

    :param max_num:
    :param items:
    :return:
    """
    step = (max_num - 1) / (items - 1)
    return [int(i * step) for i in range(items)]


async def print_shred_progress(
        cmd: schemas.ErasureCommand,
        process: asyncio.subprocess.Process
):
    """Example how to process `shred` output."""
    async for line in process.stderr:
        logger.debug(f"{cmd.command}: {line}")


async def print_badblocks_progress(
        cmd: schemas.ErasureCommand,
        process: asyncio.subprocess.Process
):
    """Example how to process `badblocks` output."""
    async for line in process.stderr:
        clean_line = line.decode('UTF-8').replace('\b', '').replace('\n', '')
        logger.debug(f"{cmd.command}: {clean_line}")

    logger.debug("OUT")


def find_text(re_expression: str, string: str) -> Optional[str]:
    """Searches for a regular expression in a string and returns the
    first match. Used to find text in the output of the commands.

    :param str re_expression:
    :param str string:
    :return:

    Example:
    >>> find_text("([0-9]+)", "123")
    """
    text = re.search(re_expression, string)
    if text:
        return text.groups()[0]
